importScripts("./common/constants.js");

function getUserToken() {
  return new Promise((resolve) => {
    chrome.storage.local.get(["userToken"], (data) => {
      const userToken = data?.userToken;
      if (userToken) {
        resolve(userToken);
      } else {
        resolve(false);
      }
    });
  });
}

function httpGet(msg, cb) {
  getUserToken().then((token) => {
    fetch(msg.url, {
      redirect: "follow",
      method: "GET",
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json",
        Authorization: `Bearer ${token}`,
      },
    })
      .then((response) => response.text())
      .then((response) => {
        cb(JSON.parse(response));
      })
      .catch((err) => {
        cb(err);
      });
  });
}

function httpPost(msg, cb) {
  const url = msg?.url;
  const payload = msg?.payload;

  getUserToken().then((userToken) => {
    fetch(`${API_BASE_URL}/${url}`, {
      method: "POST",
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json",
        Authorization: `Bearer ${userToken}`,
      },
      body: payload ?? "",
    })
      .then((res) => res.json())
      .then((data) => cb(data));
  });
}

chrome.runtime.onMessage.addListener(function (msg, sender, sendResponse) {
  if (msg.type == "httpGet") {
    httpGet(msg, sendResponse);
  } else if (msg.type == "getTabId") {
    chrome.tabs.query({ currentWindow: true, active: true }, (tabs) => {
      sendResponse(tabs[0].id);
    });
  } else if (msg.type === "httpPost") {
    httpPost(msg, sendResponse);
  } else if (msg.type === "userToken") {
    getUserToken().then((userToken) => {
      sendResponse(userToken);
    });
  }
  return true;
});

chrome.runtime.onInstalled.addListener((reason) => {
  chrome.tabs.query({}, function (tabs) {
    for (var i = 0; i < tabs.length; i++) {
      if (
        tabs[i].url.indexOf("mail.google.com") >= 0 ||
        tabs[i].url.indexOf("chat.google.com") >= 0
      ) {
        chrome.tabs.reload(tabs[i].id);
      }
    }
  });
});
