class Cursor {
  static getCurrentCursorPosition(parentElement) {
    var selection = window.getSelection(),
      charCount = -1,
      node;

    if (selection.focusNode) {
      if (Cursor._isChildOf(selection.focusNode, parentElement)) {
        node = selection.focusNode;
        charCount = selection.focusOffset;

        while (node) {
          if (node === parentElement) {
            break;
          }

          if (node.previousSibling) {
            node = node.previousSibling;
            charCount += node.textContent.length;
          } else {
            node = node.parentNode;
            if (node === null) {
              break;
            }
          }
        }
      }
    }

    return charCount;
  }

  static setCurrentCursorPosition(chars, element) {
    if (chars >= 0) {
      var selection = window.getSelection();

      let range = Cursor._createRange(element, { count: chars });

      if (range) {
        range.collapse(false);
        selection.removeAllRanges();
        selection.addRange(range);
      }
    }
  }

  static _createRange(node, chars, range) {
    if (!range) {
      range = document.createRange();
      range.selectNode(node);
      range.setStart(node, 0);
    }

    if (chars.count === 0) {
      range.setEnd(node, chars.count);
    } else if (node && chars.count > 0) {
      if (node.nodeType === Node.TEXT_NODE) {
        if (node.textContent.length < chars.count) {
          chars.count -= node.textContent.length;
        } else {
          range.setEnd(node, chars.count);
          chars.count = 0;
        }
      } else {
        for (var lp = 0; lp < node.childNodes.length; lp++) {
          range = Cursor._createRange(node.childNodes[lp], chars, range);

          if (chars.count === 0) {
            break;
          }
        }
      }
    }

    return range;
  }

  static _isChildOf(node, parentElement) {
    while (node !== null) {
      if (node === parentElement) {
        return true;
      }
      node = node.parentNode;
    }

    return false;
  }
}

function fallbackCopyTextToClipboard(text) {
  var textArea = document.createElement("textarea");
  textArea.value = text;

  // Avoid scrolling to bottom
  textArea.style.top = "0";
  textArea.style.left = "0";
  textArea.style.position = "fixed";

  document.body.appendChild(textArea);
  textArea.focus();
  textArea.select();

  try {
    var successful = document.execCommand("copy");
    var msg = successful ? "successful" : "unsuccessful";
  } catch (err) {}

  document.body.removeChild(textArea);
}

function copyToClipboard(text) {
  if (window.clipboardData && window.clipboardData.setData) {
    // Internet Explorer-specific code path to prevent textarea being shown while dialog is visible.
    return window.clipboardData.setData("Text", text);
  } else if (
    document.queryCommandSupported &&
    document.queryCommandSupported("copy")
  ) {
    var textarea = document.createElement("textarea");
    textarea.textContent = text;
    textarea.style.position = "fixed"; // Prevent scrolling to bottom of page in Microsoft Edge.
    document.body.appendChild(textarea);
    textarea.select();
    try {
      return document.execCommand("copy"); // Security exception may be thrown by some browsers.
    } catch (ex) {
      console.warn("Copy to clipboard failed.", ex);
      return false;
    } finally {
      document.body.removeChild(textarea);
    }
  }
}

function showToast(msg, delayHide) {
  $("#snackbar").addClass("show").html(msg);
  setTimeout(() => {
    $("#snackbar").removeClass("show");
  }, delayHide);
}

function showLoginModal() {
  if ($(".cp-backdrop").length === 0) {
    $("body").append(
      `<div class="cp-backdrop">
        <div class="cp-modal">
          <img
            src='${chrome.runtime.getURL("media/img/chat-plus.jpg")}'
            class="logo"
          />
          <span style="font-weight: bold;">Chat plus</span>
          <span style="margin-top: 10px;">Please login to use this add-on</span>
          <div class="google-signin-wrapper">
            <div class="google-signin-image-wrapper">
              <img
                src='${chrome.runtime.getURL("media/img/google.svg")}'
                width="25px"
                height="25px"
                style="padding: 10px;"
              />
            </div>
            <span class="google-signin-text">Sign in with Google</span>
          </div>
          <span class="cp-modal-close" id="cp-modal-close">
            ✖
          </span>
        </div>
      </div>`
    );

    $("#cp-modal-close").click(() => {
      $(".cp-backdrop").remove();
    });

    $(".google-signin-wrapper").click(() => {
      const guid = crypto.randomUUID();

      const newWindow = window.open(
        `https://accounts.google.com/o/oauth2/v2/auth?redirect_uri=${REDIRECT_URI}&prompt=consent&response_type=code&client_id=${CLIENT_ID}&scope=email+profile&access_type=offline&state=${guid}`,
        "_blank",
        "width=733,height=572,top=100,left=100"
      );

      pollingForUserTaken(newWindow, guid, (e) => console.log("closed"));
    });
  }
}

function pollingForUserTaken(window, state, cb) {
  if ($(".cp-backdrop").length === 0) {
    cb();
  } else {
    fetch(`${API_BASE_URL}/api/auth/user-token`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ state: state }),
    })
      .then((res) => res.json())
      .then((data) => {
        const { isUserloggedIn, email } = data;

        if (isUserloggedIn) {
          chrome.storage.local.set(
            { userToken: state, email: email },
            () => {}
          );
          window.close();
          $(".cp-backdrop").remove();
        }
      });
    setTimeout(() => pollingForUserTaken(window, state, cb), 2000);
  }
}

function pasteHtmlAtCaret(html, isSelection) {
  var sel, range;
  if (window.getSelection) {
    // IE9 and non-IE
    sel = window.getSelection();

    if (sel.getRangeAt && sel.rangeCount) {
      range = sel.getRangeAt(0);
      range.deleteContents();

      // Range.createContextualFragment() would be useful here but is
      // non-standard and not supported in all browsers (IE9, for one)
      var el = document.createElement("div");

      el.innerHTML = html;
      var frag = document.createDocumentFragment(),
        node,
        lastNode;
      while ((node = el.firstChild)) {
        lastNode = frag.appendChild(node);
      }

      range.insertNode(frag);

      if (isSelection) {
        // Preserve the selection

        range = range.cloneRange();
        range.setStartAfter(lastNode);
        range.collapse(true);
        sel.removeAllRanges();
        sel.addRange(range);
      }
    }
  } else if (document.selection && document.selection.type != "Control") {
    // IE < 9
    document.selection.createRange().pasteHTML(html);
  }
}

// Sentence Casing
function firstLetterUpper(string) {
  var newString = string
    .toLowerCase()
    .replace(/(^\s*\w|[\.\!\?]\s*\w)/g, function (c) {
      return c.toUpperCase();
    });
  return newString;
}

function convertToSentenceCase(string) {
  var newText = firstLetterUpper(string);
  return newText;
}

// Getting caret parent element
function getCaretParentNode() {
  var target = null;
  if (window.getSelection) {
    target = window.getSelection().getRangeAt(0).commonAncestorContainer;
    return target.nodeType === 1 ? target : target.parentNode;
  } else if (document.selection) {
    var target = document.selection.createRange().parentElement();
  }
  return target;
}

// Paste event
function onPaste(e) {
  if (getCaretParentNode().tagName === "PRE") {
    var content;
    e.preventDefault();

    if (e.originalEvent.clipboardData) {
      content = (e.originalEvent || e).clipboardData.getData("text/plain");
      pasteHtmlAtCaret(content.replace("<", "&lt;"), false);
      //document.execCommand("insertText", false, content);
      return false;
    } else if (window.clipboardData) {
      content = window.clipboardData.getData("Text");
      if (window.getSelection)
        window
          .getSelection()
          .getRangeAt(0)
          .insertNode(document.createTextNode(content));
    }
  }
}

// Adding analytics
function addAnalytics(featureName) {
  chrome.runtime.sendMessage(
    {
      type: "httpPost",
      url: "api/analytics/add",
      payload: JSON.stringify({
        featureName,
      }),
    },
    () => {}
  );
}

// Preserve Selection
const save = function () {
  const selection = window.getSelection();
  return selection.rangeCount === 0 ? null : selection.getRangeAt(0);
};

// Restore the selection
// `range` is a `Range` object
const restore = function (range) {
  const selection = window.getSelection();
  selection.removeAllRanges();
  selection.addRange(range);
};

function inIframe() {
  try {
    return window.self !== window.top;
  } catch (e) {
    return true;
  }
}
