const locationBtn = document.querySelector("#location");
const leafletMap = document.querySelector("#map");
const send = document.querySelector("#send");
const cancel = document.querySelector("#cancel");
let marker;
let map;
let cords = { lat: 51.4853489, long: -0.0686518 };

const getLocation = () => {
  if (navigator.geolocation) {
    navigator.geolocation.getCurrentPosition((position) => {
      let lat = position.coords.latitude;
      let long = position.coords.longitude;

      marker.setLatLng([lat, long]).update();

      map.flyTo([lat, long], 17, {
        animate: true,
        duration: 1.8,
      });

      map.attributionControl.removeAttribution(
        `Lat: ${cords.lat}, Long: ${cords.long}`
      );

      cords.lat = lat;
      cords.long = long;

      map.attributionControl.addAttribution(
        `Lat: ${cords.lat}, Long: ${cords.long}`
      );
    });
  } else {
    alert("Geo location is no supported in the broswer");
  }
};

const setMap = (lat = 51.4853489, long = -0.0686518, zoom = 1.949) => {
  var osm = L.tileLayer("https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
    maxZoom: 19,
    attribution: `Lat: ${cords.lat}, Long: ${cords.long}`,
  });

  var satellite = L.tileLayer(
    "http://{s}.google.com/vt/lyrs=s&x={x}&y={y}&z={z}",
    {
      maxZoom: 18,
      subdomains: ["mt0", "mt1", "mt2", "mt3"],
      attribution: `Lat: ${cords.lat}, Long: ${cords.long}`,
    }
  );

  map = L.map("map", {
    center: [lat, long],
    zoom,
    layers: [osm],
  });

  var baseMaps = {
    Map: osm,
    Satellite: satellite,
  };

  L.control.layers(baseMaps).addTo(map);

  var markerIcon = L.icon({
    iconUrl: chrome.runtime.getURL("media/img/marker-icon.svg"),
    iconSize: [50, 50], // size of the icon
  });

  marker = L.marker([lat, long], { icon: markerIcon }).addTo(map);
  marker.setLatLng([lat, long]).update();

  map.on("click", (event) => {
    marker.setLatLng([event.latlng.lat, event.latlng.lng]).update();
    map.attributionControl.removeAttribution(
      `Lat: ${cords.lat}, Long: ${cords.long}`
    );
    cords.lat = event.latlng.lat;
    cords.long = event.latlng.lng;

    map.attributionControl.addAttribution(
      `Lat: ${cords.lat}, Long: ${cords.long}`
    );
  });
};

if (locationBtn) {
  locationBtn.addEventListener("click", getLocation);
}

if (leafletMap) {
  setMap(51.4853489, -0.0686518, 1.949);
}

if (send) {
  send.addEventListener("click", () => {
    const urlParams = new URLSearchParams(window.location.search);
    const tabId = urlParams.get("tabId");

    chrome.tabs.sendMessage(Number(tabId), { type: "location", cords });

    setTimeout(() => {
      window.close();
    }, 10);
  });
}

if (cancel) {
  cancel.addEventListener("click", () => window.close());
}
