const btn = document.querySelector("#recorder");
const playPauseBtn = document.querySelector("#play-pause-btn");
const sendBtn = document.querySelector("#send");
const cancleBtn = document.querySelector("#cancel");
const animateDiv = document.querySelector(".outer-2");
const uploadingIcon = document.querySelector(".upload-svg");
const audioSvg = document.querySelector(".icon-microphone");
const recordText = document.querySelector(".record-text");
let isRecording = false;
let isUploading = false;
let recorder;
let audio;
let elapsedTimeIntervalRef;
let startTime;
let elapsedTime;

function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

async function stopRecording() {
  audio = await recorder.stop();
  isRecording = false;
  resetStopwatch();
  if (elapsedTime < 300) {
    sendBtn.removeAttribute("disabled");
  }
  playPauseBtn.classList.add("play-pause-btn-disabled");
  animateDiv.classList.remove("animate-recorder");
  audioSvg.innerHTML = `<svg
  fill="#ffffff"
  width="50"
  height="50"
  version="1.1"
  id="lni_lni-mic"
  xmlns="http://www.w3.org/2000/svg"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  x="0px"
  y="0px"
  viewBox="0 0 64 64"
  style="enable-background: new 0 0 64 64"
  xml:space="preserve"
>
  <g>
    <path
      d="M32,43.8c3.2,0,6.2-1.2,8.4-3.5c2.2-2.2,3.4-5.1,3.3-8.2V13c0-6.5-5.3-11.7-11.7-11.7c-6.5,0-11.7,5.3-11.7,11.7V32
        C20.2,38.5,25.5,43.8,32,43.8z M23.7,13c0-4.5,3.7-8.2,8.2-8.2c4.5,0,8.2,3.7,8.2,8.2v19.1c0,2.2-0.8,4.2-2.3,5.8
        c-1.6,1.6-3.7,2.5-5.9,2.5c-4.5,0-8.2-3.7-8.2-8.2V13z"
    />
    <path
      d="M52.7,31.5c0-1-0.8-1.8-1.8-1.8s-1.8,0.8-1.8,1.8c0,9.5-7.7,17.2-17.3,17.2c-9.5,0-17.2-7.7-17.2-17.2c0-1-0.8-1.8-1.8-1.8
        s-1.8,0.8-1.8,1.8c0,10.8,8.3,19.7,18.9,20.6v7.1h-7c-1,0-1.8,0.8-1.8,1.8s0.8,1.8,1.8,1.8h17.5c1,0,1.8-0.8,1.8-1.8
        s-0.8-1.8-1.8-1.8h-7v-7.1C44.3,51.3,52.7,42.4,52.7,31.5z"
    />
  </g>
</svg>`;
}

let timeAndDateHandling = {
  getElapsedTime: function (startTime) {
    let endTime = new Date();
    let timeDiff = endTime.getTime() - startTime.getTime();

    timeDiff = timeDiff / 1000;
    let seconds = Math.floor(timeDiff % 60);
    elapsedTime = timeDiff;

    if (timeDiff > 300) {
      alert("Recording time execeeded");
      stopRecording();
    }

    let secondsAsString = seconds < 10 ? "0" + seconds : seconds + "";
    timeDiff = Math.floor(timeDiff / 60);
    let minutes = timeDiff % 60;
    let minutesAsString = minutes < 10 ? "0" + minutes : minutes + "";
    timeDiff = Math.floor(timeDiff / 60);
    let hours = timeDiff % 24;
    timeDiff = Math.floor(timeDiff / 24);
    let days = timeDiff;

    let totalHours = hours + days * 24;
    let totalHoursAsString =
      totalHours < 10 ? "0" + totalHours : totalHours + "";

    if (totalHoursAsString === "00") {
      return minutesAsString + ":" + secondsAsString;
    } else {
      return totalHoursAsString + ":" + minutesAsString + ":" + secondsAsString;
    }
  },
};

function startStopwatch() {
  startTime = new Date();
  elapsedTimeIntervalRef = setInterval(() => {
    recordText.innerText = timeAndDateHandling.getElapsedTime(startTime);
  }, 1000);
}

/** Resets stopwatch */
function resetStopwatch() {
  if (typeof elapsedTimeIntervalRef !== "undefined") {
    clearInterval(elapsedTimeIntervalRef);
    elapsedTimeIntervalRef = undefined;
  }
  recordText.innerText = "";
}

if (audioSvg) {
  audioSvg.innerHTML = ` <svg
  fill="#ffffff"
  width="50"
  height="50"
  version="1.1"
  id="lni_lni-mic"
  xmlns="http://www.w3.org/2000/svg"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  x="0px"
  y="0px"
  viewBox="0 0 64 64"
  style="enable-background: new 0 0 64 64"
  xml:space="preserve"
>
  <g>
    <path
      d="M32,43.8c3.2,0,6.2-1.2,8.4-3.5c2.2-2.2,3.4-5.1,3.3-8.2V13c0-6.5-5.3-11.7-11.7-11.7c-6.5,0-11.7,5.3-11.7,11.7V32
        C20.2,38.5,25.5,43.8,32,43.8z M23.7,13c0-4.5,3.7-8.2,8.2-8.2c4.5,0,8.2,3.7,8.2,8.2v19.1c0,2.2-0.8,4.2-2.3,5.8
        c-1.6,1.6-3.7,2.5-5.9,2.5c-4.5,0-8.2-3.7-8.2-8.2V13z"
    />
    <path
      d="M52.7,31.5c0-1-0.8-1.8-1.8-1.8s-1.8,0.8-1.8,1.8c0,9.5-7.7,17.2-17.3,17.2c-9.5,0-17.2-7.7-17.2-17.2c0-1-0.8-1.8-1.8-1.8
        s-1.8,0.8-1.8,1.8c0,10.8,8.3,19.7,18.9,20.6v7.1h-7c-1,0-1.8,0.8-1.8,1.8s0.8,1.8,1.8,1.8h17.5c1,0,1.8-0.8,1.8-1.8
        s-0.8-1.8-1.8-1.8h-7v-7.1C44.3,51.3,52.7,42.4,52.7,31.5z"
    />
  </g>
</svg>`;
}

if (playPauseBtn) {
  playPauseBtn.addEventListener("click", async () => {
    if (isRecording) {
      await stopRecording();
    } else {
      sendBtn.setAttribute("disabled", "");

      if (!recorder) {
        recorder = await recorderAudio();
      }

      const audioElement = document.querySelector("#audio");
      if (audioElement) {
        audioElement.remove();
      }

      playPauseBtn.classList.remove("play-pause-btn-disabled");
      startStopwatch();
      recorder.start();
      isRecording = true;
      audioSvg.innerHTML = `<svg
      xmlns="http://www.w3.org/2000/svg"
      height="48"
      fill="#ffffff"
      width="48"
      xmlns="http://www.w3.org/2000/svg"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      x="0px"
      y="0px"
    >
      <path
        d="M16.5 31.5h15v-15h-15ZM24 44q-4.1 0-7.75-1.575-3.65-1.575-6.375-4.3-2.725-2.725-4.3-6.375Q4 28.1 4 23.95q0-4.1 1.575-7.75 1.575-3.65 4.3-6.35 2.725-2.7 6.375-4.275Q19.9 4 24.05 4q4.1 0 7.75 1.575 3.65 1.575 6.35 4.275 2.7 2.7 4.275 6.35Q44 19.85 44 24q0 4.1-1.575 7.75-1.575 3.65-4.275 6.375t-6.35 4.3Q28.15 44 24 44Zm.05-3q7.05 0 12-4.975T41 23.95q0-7.05-4.95-12T24 7q-7.05 0-12.025 4.95Q7 16.9 7 24q0 7.05 4.975 12.025Q16.95 41 24.05 41ZM24 24Z"
      />
    </svg>`;
    }
  });
}

if (sendBtn) {
  sendBtn.addEventListener("click", () => {
    uploadingIcon.classList.add("upload-svg-visible");
    const reader = new FileReader();
    reader.readAsDataURL(audio.audioBlob);
    reader.onload = () => {
      const base64AudioMessage = reader.result.split(",")[1];

      chrome.runtime.sendMessage(
        {
          type: "httpPost",
          url: "api/audio/upload",
          payload: JSON.stringify({ message: base64AudioMessage }),
        },
        (response) => {
          const { message, status } = response;

          if (status === "success") {
            const urlParams = new URLSearchParams(window.location.search);
            const tabId = urlParams.get("tabId");

            chrome.tabs.sendMessage(Number(tabId), {
              type: "audio",
              message,
            });
            uploadingIcon.classList.remove("upload-svg-visible");
            setTimeout(() => {
              window.close();
            }, 10);
          }
        }
      );
    };
  });
}

if (cancleBtn) {
  cancleBtn.addEventListener("click", () => {
    window.close();
  });
}

const recorderAudio = () => {
  return new Promise((resolve) => {
    navigator.mediaDevices.getUserMedia({ audio: true }).then((stream) => {
      const mediaRecorder = new MediaRecorder(stream);
      let chunks = [];

      mediaRecorder.addEventListener("dataavailable", (event) => {
        if (event.data.size > 400) {
          if (!animateDiv.classList.contains("animate-recorder")) {
            animateDiv.classList.add("animate-recorder");
          }
        } else {
          animateDiv.classList.remove("animate-recorder");
        }
        chunks.push(event.data);
      });

      const start = () => {
        chunks = [];
        mediaRecorder.start(1);
      };

      const stop = () => {
        return new Promise((resolve) => {
          mediaRecorder.addEventListener("stop", () => {
            const audioBlob = new Blob(chunks, { type: "audio/mp3" });

            if (!document.querySelector("audio")) {
              const audioUrl = URL.createObjectURL(audioBlob);
              const audioElement = document.createElement("audio");
              audioElement.setAttribute("id", "audio");
              audioElement.setAttribute("controls", "");
              audioElement.style.marginTop = "30px";
              audioElement.innerHTML = `<source
            src="${audioUrl}"
            type="audio/mp3"
          />`;

              insertAfter(playPauseBtn, audioElement);
            }

            resolve({ audioBlob });
          });

          mediaRecorder.stop();
        });
      };

      resolve({ start, stop });
    });
  });
};
